@@ -27,7 +27,7 @@
 #include <miopen/conv/heuristics/ai_heuristics.hpp>
 #if MIOPEN_ENABLE_AI_IMMED_MODE_FALLBACK || MIOPEN_ENABLE_AI_KERNEL_TUNING
 #include <fdeep/fdeep.hpp>
-#include <boost/filesystem.hpp>
+#include <miopen/filesystem.hpp>
 
 namespace miopen {
 namespace ai {
@@ -35,7 +35,7 @@ namespace common {
 
 nlohmann::json LoadJSON(const std::string& path)
 {
-    if(!boost::filesystem::exists(path))
+    if(!fs::exists(path))
         MIOPEN_THROW(miopenStatusInternalError, "Unable to load file: " + path);
     return nlohmann::json::parse(std::ifstream(path));
 }
@@ -119,10 +119,10 @@ public:
           offset(metadata.num_outputs - metadata.num_solvers)
     {
     }
-    virtual ~Model()                                                     = default;
-    virtual bool IsProblemSupported(const ProblemDescription& problem,
-                                    const ConvolutionContext& ctx) const = 0;
-    std::vector<float> Forward(const ProblemDescription& problem) const
+    virtual ~Model()                                                   = default;
+    virtual bool IsProblemSupported(const conv::ProblemDescription& problem,
+                                    const ExecutionContext& ctx) const = 0;
+    std::vector<float> Forward(const conv::ProblemDescription& problem) const
     {
         std::vector<float> features       = ToFeatures(problem);
         std::vector<fdeep::tensor> output = model.predict({fdeep::tensor(input_shape, features)});
@@ -138,58 +138,57 @@ protected:
     static std::string ModelPath(const std::string& arch)
     {
         const auto file_path = GetSystemDbPath() + "/" + arch + ".tn.model";
-        if(!boost::filesystem::exists(file_path))
+        if(!fs::exists(file_path))
             MIOPEN_THROW(miopenStatusInternalError, "Unable to load AI model file:" + file_path);
         return file_path;
     }
-    virtual std::vector<float> ToFeatures(const ProblemDescription& problem) const = 0;
+    virtual std::vector<float> ToFeatures(const conv::ProblemDescription& problem) const = 0;
 };
 
-class Gfx908Model : public Model
+class Gfx908Model final : public Model
 {
 public:
     Gfx908Model() : Model("gfx908") {}
-    bool IsProblemSupported(const ProblemDescription& problem,
-                            const ConvolutionContext& ctx) const override
+    bool IsProblemSupported(const conv::ProblemDescription& problem,
+                            const ExecutionContext& ctx) const override
     {
         // check if problem is of the kind TunaNet was trained to handle
-        if(!problem.conv_problem.Is2d())
+        if(!problem.Is2d())
         {
             MIOPEN_LOG_I2("TunaNet Inapplicable: Problem not 2D");
             return false;
         }
-        if(problem.conv_problem.GetGroupCount() != 1)
+        if(problem.GetGroupCount() != 1)
         {
             MIOPEN_LOG_I2("TunaNet Inapplicable: Group count not 1");
             return false;
         }
-        if(problem.conv_problem.GetInLayout() != "NCHW" &&
-           problem.conv_problem.GetInLayout() != "NCDHW")
+        if(problem.GetInLayout() != "NCHW" && problem.GetInLayout() != "NCDHW")
         {
             MIOPEN_LOG_I2("TunaNet Inapplicable: Layout not supported");
             return false;
         }
-        if(problem.conv_problem.GetWeightsHeight() != problem.conv_problem.GetWeightsWidth())
+        if(problem.GetWeightsHeight() != problem.GetWeightsWidth())
         {
             MIOPEN_LOG_I2("TunaNet Inapplicable: Filters must be square (fil_h == fil_w)");
             return false;
         }
-        if(problem.conv_problem.GetPadH() != problem.conv_problem.GetPadW())
+        if(problem.GetPadH() != problem.GetPadW())
         {
             MIOPEN_LOG_I2("TunaNet Inapplicable: Padding must be equal along all axes");
             return false;
         }
-        if(problem.conv_problem.GetKernelStrideH() != problem.conv_problem.GetKernelStrideW())
+        if(problem.GetKernelStrideH() != problem.GetKernelStrideW())
         {
             MIOPEN_LOG_I2("TunaNet Inapplicable: Stride must be equal along all axes");
             return false;
         }
-        if(problem.conv_problem.GetDilationH() != 1 || problem.conv_problem.GetDilationW() != 1)
+        if(problem.GetDilationH() != 1 || problem.GetDilationW() != 1)
         {
             MIOPEN_LOG_I2("TunaNet Inapplicable: Dilation must be 1");
             return false;
         }
-        const auto& data_type = problem.conv_problem.GetInDataType();
+        const auto data_type = problem.GetInDataType();
         if(data_type != miopenFloat && data_type != miopenHalf && data_type != miopenBFloat16)
         {
             MIOPEN_LOG_I2("TunaNet Inapplicable: Unsupported data type");
@@ -217,39 +216,36 @@ public:
     }
 
 protected:
-    std::vector<float> ToFeatures(const ProblemDescription& problem) const override
+    std::vector<float> ToFeatures(const conv::ProblemDescription& problem) const override
     {
-        const auto& conv_problem    = problem.conv_problem;
-        const bool isFwd            = conv_problem.GetDirection() == conv::Direction::Forward;
+        const bool isFwd            = problem.GetDirection() == conv::Direction::Forward;
         std::vector<float> features = {
-            static_cast<float>(isFwd ? conv_problem.GetInChannels()
-                                     : conv_problem.GetOutChannels()),
-            static_cast<float>(isFwd ? conv_problem.GetInDepth() : conv_problem.GetOutDepth()),
-            static_cast<float>(isFwd ? conv_problem.GetInHeight() : conv_problem.GetOutHeight()),
-            static_cast<float>(isFwd ? conv_problem.GetInWidth() : conv_problem.GetOutWidth()),
-            static_cast<float>(conv_problem.GetWeightsDepth()),
-            static_cast<float>(conv_problem.GetWeightsHeight()),
-            static_cast<float>(conv_problem.GetWeightsWidth()),
-            static_cast<float>(isFwd ? conv_problem.GetOutChannels()
-                                     : conv_problem.GetInChannels()),
-            static_cast<float>(isFwd ? conv_problem.GetOutDepth() : conv_problem.GetInDepth()),
-            static_cast<float>(isFwd ? conv_problem.GetOutHeight() : conv_problem.GetInHeight()),
-            static_cast<float>(isFwd ? conv_problem.GetOutWidth() : conv_problem.GetInWidth()),
-            static_cast<float>(conv_problem.GetOutBatchSize()),
+            static_cast<float>(isFwd ? problem.GetInChannels() : problem.GetOutChannels()),
+            static_cast<float>(isFwd ? problem.GetInDepth() : problem.GetOutDepth()),
+            static_cast<float>(isFwd ? problem.GetInHeight() : problem.GetOutHeight()),
+            static_cast<float>(isFwd ? problem.GetInWidth() : problem.GetOutWidth()),
+            static_cast<float>(problem.GetWeightsDepth()),
+            static_cast<float>(problem.GetWeightsHeight()),
+            static_cast<float>(problem.GetWeightsWidth()),
+            static_cast<float>(isFwd ? problem.GetOutChannels() : problem.GetInChannels()),
+            static_cast<float>(isFwd ? problem.GetOutDepth() : problem.GetInDepth()),
+            static_cast<float>(isFwd ? problem.GetOutHeight() : problem.GetInHeight()),
+            static_cast<float>(isFwd ? problem.GetOutWidth() : problem.GetInWidth()),
+            static_cast<float>(problem.GetOutBatchSize()),
             static_cast<float>(1), // TunaNet was trained on a dataset of 2D
                                    // problems where PadD was incorrectly set to 1
-            static_cast<float>(conv_problem.GetPadH()),
-            static_cast<float>(conv_problem.GetPadW()),
+            static_cast<float>(problem.GetPadH()),
+            static_cast<float>(problem.GetPadW()),
             static_cast<float>(1), // TunaNet was trained on a dataset of 2D
                                    // problems where StrideD was incorrectly set to 1
-            static_cast<float>(conv_problem.GetKernelStrideH()),
-            static_cast<float>(conv_problem.GetKernelStrideW()),
-            static_cast<float>(conv_problem.GetDilationH()),
-            static_cast<float>(conv_problem.GetDilationW()),
-            static_cast<float>(metadata.EncodeLayout(conv_problem.GetInLayout())),
-            static_cast<float>(metadata.EncodePrecision(conv_problem.GetInDataType())),
-            static_cast<float>(metadata.EncodeDirection(conv_problem.GetDirection())),
-            static_cast<float>(conv_problem.GetGroupCount())};
+            static_cast<float>(problem.GetKernelStrideH()),
+            static_cast<float>(problem.GetKernelStrideW()),
+            static_cast<float>(problem.GetDilationH()),
+            static_cast<float>(problem.GetDilationW()),
+            static_cast<float>(metadata.EncodeLayout(problem.GetInLayout())),
+            static_cast<float>(metadata.EncodePrecision(problem.GetInDataType())),
+            static_cast<float>(metadata.EncodeDirection(problem.GetDirection())),
+            static_cast<float>(problem.GetGroupCount())};
 
         // normalize
         for(size_t i = 0; i < features.size(); ++i)
@@ -259,10 +255,109 @@ protected:
     }
 };
 
-std::unique_ptr<Model> GetModel(const std::string&) { return std::make_unique<Gfx908Model>(); }
+class Gfx90aModel final : public Model
+{
+public:
+    Gfx90aModel() : Model("gfx90a") {}
+    bool IsProblemSupported(const conv::ProblemDescription& problem,
+                            const ExecutionContext& ctx) const override
+    {
+        // check if problem is of the kind TunaNet was trained to handle
+        if(!problem.Is2d())
+        {
+            MIOPEN_LOG_I2("TunaNet Inapplicable: Problem not 2D");
+            return false;
+        }
+        if(problem.GetInLayout() != "NCHW")
+        {
+            MIOPEN_LOG_I2("TunaNet Inapplicable: Layout not supported");
+            return false;
+        }
+        if(problem.GetKernelStrideH() != problem.GetKernelStrideW())
+        {
+            MIOPEN_LOG_I2("TunaNet Inapplicable: Stride must be equal along all axes");
+            return false;
+        }
+        if(problem.GetDilationH() != problem.GetDilationW())
+        {
+            MIOPEN_LOG_I2("TunaNet Inapplicable: Dilation must be 1");
+            return false;
+        }
+        if(problem.GetBias() != 0)
+        {
+            MIOPEN_LOG_I2("TunaNet Inapplicable: Bias must be 0");
+            return false;
+        }
+        const auto data_type = problem.GetInDataType();
+        if(data_type != miopenFloat && data_type != miopenHalf && data_type != miopenBFloat16)
+        {
+            MIOPEN_LOG_I2("TunaNet Inapplicable: Unsupported data type");
+            return false;
+        }
 
-std::vector<uint64_t> PredictSolver(const ProblemDescription& problem,
-                                    const ConvolutionContext& ctx,
+        // check if the context is s.t. no solver TunaNet may predict would be applicable
+        size_t applicable_solvers = 0;
+        for(const auto& solver_name : metadata.solver_map)
+        {
+            auto solver_id = solver::Id{solver_name.second};
+            auto solver    = solver_id.GetSolver();
+            if(solver.IsApplicable(ctx, problem))
+            {
+                applicable_solvers++;
+                break;
+            }
+        }
+        if(applicable_solvers == 0)
+        {
+            MIOPEN_LOG_I2("TunaNet Inapplicable: No solver that TunaNet may predict applies");
+            return false;
+        }
+        MIOPEN_LOG_I2("TunaNet Applicable");
+        return true;
+    }
+
+protected:
+    std::vector<float> ToFeatures(const conv::ProblemDescription& problem) const override
+    {
+        const bool isFwd            = problem.GetDirection() == conv::Direction::Forward;
+        std::vector<float> features = {
+            static_cast<float>(isFwd ? problem.GetInChannels() : problem.GetOutChannels()),
+            static_cast<float>(isFwd ? problem.GetInHeight() : problem.GetOutHeight()),
+            static_cast<float>(isFwd ? problem.GetInWidth() : problem.GetOutWidth()),
+            static_cast<float>(isFwd ? problem.GetOutChannels() : problem.GetInChannels()),
+            static_cast<float>(isFwd ? problem.GetOutHeight() : problem.GetInHeight()),
+            static_cast<float>(isFwd ? problem.GetOutWidth() : problem.GetInWidth()),
+            static_cast<float>(problem.GetWeightsHeight()),
+            static_cast<float>(problem.GetWeightsWidth()),
+            static_cast<float>(problem.GetPadH()),
+            static_cast<float>(problem.GetPadW()),
+            static_cast<float>(problem.GetKernelStrideH()),
+            static_cast<float>(problem.GetKernelStrideW()),
+            static_cast<float>(problem.GetDilationH()),
+            static_cast<float>(problem.GetDilationW()),
+            static_cast<float>(problem.GetOutBatchSize()),
+            static_cast<float>(metadata.EncodePrecision(problem.GetInDataType())),
+            static_cast<float>(metadata.EncodeDirection(problem.GetDirection())),
+            static_cast<float>(problem.GetGroupCount())};
+
+        // normalize
+        for(size_t i = 0; i < features.size(); ++i)
+            features[i] = (features[i] - metadata.features_mean[i]) / metadata.features_std[i];
+
+        return features;
+    }
+};
+
+std::unique_ptr<Model> GetModel(const std::string& device)
+{
+    if(device == "gfx90a")
+        return std::make_unique<Gfx90aModel>();
+    else
+        return std::make_unique<Gfx908Model>();
+}
+
+std::vector<uint64_t> PredictSolver(const conv::ProblemDescription& problem,
+                                    const ExecutionContext& ctx,
                                     const std::string& device)
 {
     const static std::unique_ptr<Model> model = GetModel(device);
@@ -271,10 +366,10 @@ std::vector<uint64_t> PredictSolver(const ProblemDescription& problem,
 
     std::string est_name = ":memory:" + device;
     auto& db             = AnyRamDb::GetCached(est_name);
-    auto db_res          = db.FindRecord(problem.conv_problem);
+    auto db_res          = db.FindRecord(problem);
     if(db_res)
     {
-        MIOPEN_LOG_I2("Cached heuristic result found");
+        MIOPEN_LOG_I2("Cached heuristic (TunaNet) result found");
         std::vector<uint64_t> db_sol(db_res->size());
         // cast returned record to solver ids
         std::transform(db_res->begin(), db_res->end(), db_sol.begin(), [](boost::any id) {
@@ -290,7 +385,7 @@ std::vector<uint64_t> PredictSolver(const ProblemDescription& problem,
         return db_sol;
     }
 
-    MIOPEN_LOG_I2("Evaluating Heuristic");
+    MIOPEN_LOG_I2("Evaluating TunaNet");
 
     std::vector<float> res = model->Forward(problem);
     std::vector<std::pair<int, float>> sort_res(res.size());
@@ -320,13 +415,13 @@ std::vector<uint64_t> PredictSolver(const ProblemDescription& problem,
         sol.push_back(sol_id.Value());
         any_sol.push_back(sol_id.Value());
     }
-    db.StoreRecord(problem.conv_problem, any_sol);
+    db.StoreRecord(problem, any_sol);
     if(miopen::IsLogging(LoggingLevel::Info2))
     {
         std::stringstream ss;
         for(auto& id : sol)
             ss << solver::Id{id}.ToString() << " ID:" << id << ", ";
-        MIOPEN_LOG_I2("Heuristic Result: " << ss.str());
+        MIOPEN_LOG_I2("TunaNet Result: " << ss.str());
     }
     return sol;
 }
@@ -341,7 +436,8 @@ Metadata::Metadata(const std::string& arch, const std::string& solver)
     const nlohmann::json metadata =
         common::LoadJSON(GetSystemDbPath() + "/" + arch + "_" + solver + "_metadata.ktn.model");
     num_tuning_params = metadata["num_tuning_params"].get<std::size_t>();
-    tuning_decodings = metadata["decodings"]["tunings"].get<std::unordered_map<std::string, int>>();
+    tuning_decodings =
+        metadata["decodings"]["tunings"].get<std::unordered_map<std::string, std::string>>();
 }
 
 class Model
@@ -355,9 +451,11 @@ public:
     {
     }
     virtual ~Model() = default;
-    fdeep::tensors Encode(const std::vector<float>& features, std::size_t dim) const
+    fdeep::tensors Encode(const std::vector<float>& features, std::size_t dim, bool transform) const
     {
-        fdeep::tensor input_tensor = fdeep::tensor(fdeep::tensor_shape(dim, dim), features);
+        const auto tensor_shape_depth = transform ? dim : 1;
+        fdeep::tensor input_tensor =
+            fdeep::tensor(fdeep::tensor_shape(dim, tensor_shape_depth), features);
         return encoder.predict({input_tensor});
     }
     fdeep::tensors Decode(const float prev_token, const fdeep::tensors& context) const
@@ -377,7 +475,7 @@ private:
     {
         const std::string path =
             GetSystemDbPath() + "/" + arch + "_" + solver + "_encoder.ktn.model";
-        if(!boost::filesystem::exists(path))
+        if(!fs::exists(path))
             MIOPEN_THROW(miopenStatusInternalError, "Unable to load file: " + path);
         return path;
     }
@@ -385,7 +483,7 @@ private:
     {
         const std::string path =
             GetSystemDbPath() + "/" + arch + "_" + solver + "_decoder.ktn.model";
-        if(!boost::filesystem::exists(path))
+        if(!fs::exists(path))
             MIOPEN_THROW(miopenStatusInternalError, "Unable to load file: " + path);
         return path;
     }
@@ -393,10 +491,6 @@ private:
 
 std::shared_ptr<Model> GetModel(const std::string& arch, const std::string& solver)
 {
-    static const std::string prevArch{arch};
-
-    if(prevArch != arch)
-        MIOPEN_THROW("Cannot use AI tuning models for multiple gpu architectures");
     static std::map<std::string, std::shared_ptr<Model>> models;
     auto it = models.find(solver);
     if(it == models.end())
@@ -414,11 +508,16 @@ std::shared_ptr<Model> GetModel(const std::string& arch, const std::string& solv
 bool ModelSetParams(const std::string& arch,
                     const std::string& solver,
                     const std::vector<float>& features,
-                    std::function<bool(int, int)> validator)
+                    bool transform_features,
+                    std::function<bool(std::size_t, std::string)> validator)
 {
-    auto model             = GetModel(arch, solver);
-    int dim                = std::sqrt(features.size());
-    fdeep::tensors context = model->Encode(features, dim);
+    auto model = GetModel(arch, solver);
+    int dim    = 0;
+    if(transform_features)
+        dim = std::sqrt(features.size());
+    else
+        dim = features.size();
+    fdeep::tensors context = model->Encode(features, dim, transform_features);
     float decoder_input    = 0.0;
     for(std::size_t i = 0; i < model->metadata.num_tuning_params; ++i)
     {
@@ -434,9 +533,9 @@ bool ModelSetParams(const std::string& arch,
         {
             int token = pq.top().second;
             // convert index to token value
-            int value = model->metadata.tuning_decodings[std::to_string(token)];
+            std::string value = model->metadata.tuning_decodings[std::to_string(token)];
             pq.pop();
-            if(value < 0)
+            if(value == "-1")
                 return false;
             if(validator(i, value))
             {
