@@ -32,8 +32,9 @@
 #include <miopen/find_controls.hpp>
 #include <miopen/handle.hpp>
 #include <miopen/logger.hpp>
-#include <miopen/problem_description.hpp>
+#include <miopen/conv/problem_description.hpp>
 #include <miopen/tensor_ops.hpp>
+#include <miopen/driver_arguments.hpp>
 
 #include <algorithm>
 #include <optional>
@@ -55,24 +56,16 @@ static inline auto MakeFwdCtxAndProblem(miopenHandle_t handle,
     const auto direction =
         (conv.mode != miopenTranspose) ? Direction::Forward : Direction::BackwardData;
 
-    auto problem = (conv.mode != miopenTranspose) ? ProblemDescription{miopen::deref(xDesc),
-                                                                       miopen::deref(wDesc),
-                                                                       miopen::deref(yDesc),
-                                                                       conv,
-                                                                       direction}
-                                                  : ProblemDescription{miopen::deref(yDesc),
-                                                                       miopen::deref(wDesc),
-                                                                       miopen::deref(xDesc),
-                                                                       conv,
-                                                                       direction};
-
-    auto ctx = [&] {
-        auto tmp = ExecutionContext{&miopen::deref(handle)};
-        tmp.DetectRocm();
-        problem.SetupFloats(tmp);
-        return tmp;
-    }();
+    /// \anchor transpose_convolutions_x_y_swapping
+    /// In transpose mode we exchange x with y. From the other hand, when Backward*
+    /// ProblemDescription instances are constructed, x and y shall be swapped as well.
+    /// As transpose mode swaps Forward with Backward AND x with y, the order of
+    /// ctor arguments remains the same.
+    auto problem = ProblemDescription{
+        miopen::deref(xDesc), miopen::deref(wDesc), miopen::deref(yDesc), conv, direction};
 
+    auto ctx = ExecutionContext{&miopen::deref(handle)};
+    problem.SetupFloats(ctx);
     return std::make_tuple(std::move(ctx), std::move(problem));
 }
 
@@ -86,24 +79,12 @@ static inline auto MakeBwdCtxAndProblem(miopenHandle_t handle,
     const auto direction =
         (conv.mode != miopenTranspose) ? Direction::BackwardData : Direction::Forward;
 
-    auto problem = (conv.mode != miopenTranspose) ? ProblemDescription{miopen::deref(dyDesc),
-                                                                       miopen::deref(wDesc),
-                                                                       miopen::deref(dxDesc),
-                                                                       conv,
-                                                                       direction}
-                                                  : ProblemDescription{miopen::deref(dxDesc),
-                                                                       miopen::deref(wDesc),
-                                                                       miopen::deref(dyDesc),
-                                                                       conv,
-                                                                       direction};
-
-    auto ctx = [&] {
-        auto tmp = ExecutionContext{&miopen::deref(handle)};
-        tmp.DetectRocm();
-        problem.SetupFloats(tmp);
-        return tmp;
-    }();
+    /// \ref transpose_convolutions_x_y_swapping
+    auto problem = ProblemDescription{
+        miopen::deref(dyDesc), miopen::deref(wDesc), miopen::deref(dxDesc), conv, direction};
 
+    auto ctx = ExecutionContext{&miopen::deref(handle)};
+    problem.SetupFloats(ctx);
     return std::make_tuple(std::move(ctx), std::move(problem));
 }
 
@@ -127,13 +108,8 @@ static inline auto MakeWrWCtxAndProblem(miopenHandle_t handle,
                                                                        conv,
                                                                        direction};
 
-    auto ctx = [&] {
-        auto tmp = ExecutionContext{&miopen::deref(handle)};
-        tmp.DetectRocm();
-        problem.SetupFloats(tmp);
-        return tmp;
-    }();
-
+    auto ctx = ExecutionContext{&miopen::deref(handle)};
+    problem.SetupFloats(ctx);
     return std::make_tuple(std::move(ctx), std::move(problem));
 }
 
@@ -187,6 +163,13 @@ extern "C" miopenStatus_t miopenInitConvolutionNdDescriptor(miopenConvolutionDes
     });
 }
 
+extern "C" miopenStatus_t miopenGetConvolutionGroupCount(miopenConvolutionDescriptor_t convDesc,
+                                                         int* groupCount)
+{
+    MIOPEN_LOG_FUNCTION(convDesc);
+    return miopen::try_([&] { miopen::deref(groupCount) = miopen::deref(convDesc).group_count; });
+}
+
 extern "C" miopenStatus_t miopenSetConvolutionGroupCount(miopenConvolutionDescriptor_t convDesc,
                                                          int groupCount)
 {
@@ -214,8 +197,8 @@ extern "C" miopenStatus_t miopenGetConvolutionFindMode(const miopenConvolutionDe
 }
 
 // Hidden C++ functions for MIGraphX.
-extern "C" miopenStatus_t miopenHiddenSetConvolutionFindMode(miopenConvolutionDescriptor_t convDesc,
-                                                             int findMode)
+extern "C" MIOPEN_EXPORT miopenStatus_t
+miopenHiddenSetConvolutionFindMode(miopenConvolutionDescriptor_t convDesc, int findMode)
 {
     return miopen::try_([&] {
         miopen::deref(convDesc).findMode.Set(static_cast<miopen::FindMode::Values>(findMode));
@@ -271,7 +254,7 @@ extern "C" miopenStatus_t miopenGetConvolutionDescriptor(miopenConvolutionDescri
                                                          int* dilation_h,
                                                          int* dilation_w)
 {
-    MIOPEN_LOG_FUNCTION(convDesc, c_mode, pad_h, pad_w, stride_h, stride_w, dilation_h, dilation_w);
+    MIOPEN_LOG_FUNCTION(convDesc);
     return miopen::try_([&] {
         if(miopen::deref(convDesc).GetSpatialDimension() != 2)
         {
@@ -296,8 +279,7 @@ extern "C" miopenStatus_t miopenGetConvolutionNdDescriptor(miopenConvolutionDesc
                                                            int* dilationA,
                                                            miopenConvolutionMode_t* c_mode)
 {
-    MIOPEN_LOG_FUNCTION(
-        convDesc, requestedSpatialDim, spatialDim, padA, strideA, dilationA, c_mode);
+    MIOPEN_LOG_FUNCTION(convDesc, requestedSpatialDim);
     return miopen::try_([&] {
         int spatial_dim = miopen::deref(convDesc).GetSpatialDimension();
         if(spatial_dim < requestedSpatialDim)
@@ -322,7 +304,7 @@ extern "C" miopenStatus_t miopenGetConvolutionNdDescriptor(miopenConvolutionDesc
 extern "C" miopenStatus_t miopenGetConvolutionSpatialDim(miopenConvolutionDescriptor_t convDesc,
                                                          int* spatialDim)
 {
-    MIOPEN_LOG_FUNCTION(convDesc, spatialDim);
+    MIOPEN_LOG_FUNCTION(convDesc);
     return miopen::try_(
         [&] { miopen::deref(spatialDim) = miopen::deref(convDesc).GetSpatialDimension(); });
 }
@@ -336,7 +318,7 @@ miopenGetConvolutionForwardOutputDim(miopenConvolutionDescriptor_t convDesc,
                                      int* h,
                                      int* w)
 {
-    MIOPEN_LOG_FUNCTION(convDesc, inputTensorDesc, filterDesc, n, c, h, w);
+    MIOPEN_LOG_FUNCTION(convDesc, inputTensorDesc, filterDesc);
     return miopen::try_([&] {
         if(miopen::deref(convDesc).GetSpatialDimension() != 2)
         {
@@ -357,7 +339,7 @@ miopenGetConvolutionNdForwardOutputDim(miopenConvolutionDescriptor_t convDesc,
                                        int* nDim,
                                        int* outputTensorDimA)
 {
-    MIOPEN_LOG_FUNCTION(convDesc, inputTensorDesc, filterDesc, nDim, outputTensorDimA);
+    MIOPEN_LOG_FUNCTION(convDesc, inputTensorDesc, filterDesc);
     return miopen::try_([&] {
         auto out_desc = miopen::deref(convDesc).GetForwardOutputTensor(
             miopen::deref(inputTensorDesc), miopen::deref(filterDesc));
@@ -386,144 +368,19 @@ miopenConvolutionForwardGetWorkSpaceSize(miopenHandle_t handle,
                                          size_t* workSpaceSize)
 {
 
-    MIOPEN_LOG_FUNCTION(handle, wDesc, xDesc, convDesc, yDesc, workSpaceSize);
-    miopen::try_([&] {
+    MIOPEN_LOG_FUNCTION(handle, wDesc, xDesc, convDesc, yDesc);
+    return miopen::try_([&] {
         auto ctx               = ExecutionContext{};
         auto problem           = ProblemDescription{};
         std::tie(ctx, problem) = MakeFwdCtxAndProblem(handle, xDesc, wDesc, convDesc, yDesc);
         *workSpaceSize         = miopen::deref(convDesc).GetWorkSpaceSize(ctx, problem);
     });
-
-    return (miopenStatusSuccess);
-}
-
-enum class ConvDirection
-{
-    Fwd = 1,
-    Bwd = 2,
-    WrW = 4
-};
-
-// Not static because Find 2.0 uses it until a separate driver command is implemented
-// We use API type in the declaration so that usages won't have to convert to commandline values
-static std::string ConvArgsForMIOpenDriver(const miopen::TensorDescriptor& xDesc,
-                                           const miopen::TensorDescriptor& wDesc,
-                                           const miopen::ConvolutionDescriptor& convDesc,
-                                           const miopen::TensorDescriptor& yDesc,
-                                           miopenProblemDirection_t dir,
-                                           std::optional<uint64_t> immediate_mode_solver_id)
-{
-    const auto conv_dir = [&]() {
-        switch(dir)
-        {
-        case miopenProblemDirectionForward: return ConvDirection::Fwd;
-        case miopenProblemDirectionBackward: return ConvDirection::Bwd;
-        case miopenProblemDirectionBackwardWeights: return ConvDirection::WrW;
-        }
-    }();
-
-    std::stringstream ss;
-    if(xDesc.GetType() == miopenHalf)
-    {
-        ss << "convfp16";
-    }
-    else if(xDesc.GetType() == miopenBFloat16)
-    {
-        ss << "convbfp16";
-    }
-    else if(xDesc.GetType() == miopenInt8 || xDesc.GetType() == miopenInt8x4)
-    {
-        ss << "convint8";
-    }
-    else
-    {
-        ss << "conv";
-    }
-    if(convDesc.GetSpatialDimension() == 2)
-    {
-        ss << " -n " << xDesc.GetLengths()[0] // clang-format off
-            << " -c " << xDesc.GetLengths()[1]
-            << " -H " << xDesc.GetLengths()[2]
-            << " -W " << xDesc.GetLengths()[3]
-            << " -k " << wDesc.GetLengths()[0]
-            << " -y " << wDesc.GetLengths()[2]
-            << " -x " << wDesc.GetLengths()[3]
-            << " -p " << convDesc.GetConvPads()[0]
-            << " -q " << convDesc.GetConvPads()[1]
-            << " -u " << convDesc.GetConvStrides()[0]
-            << " -v " << convDesc.GetConvStrides()[1]
-            << " -l " << convDesc.GetConvDilations()[0]
-            << " -j " << convDesc.GetConvDilations()[1]; // clang-format on
-        std::string x_layout = xDesc.GetLayout("NCHW");
-        std::string w_layout = wDesc.GetLayout("NCHW");
-        std::string y_layout = yDesc.GetLayout("NCHW");
-        if(x_layout != "NCHW")
-        {
-            ss << " --in_layout " << x_layout;
-        }
-        if(w_layout != "NCHW")
-        {
-            ss << " --fil_layout " << w_layout;
-        }
-        if(y_layout != "NCHW")
-        {
-            ss << " --out_layout " << y_layout;
-        }
-    }
-    else if(convDesc.GetSpatialDimension() == 3)
-    {
-        ss << " -n " << xDesc.GetLengths()[0] // clang-format off
-            << " -c " << xDesc.GetLengths()[1]
-            << " --in_d " << xDesc.GetLengths()[2]
-            << " -H " << xDesc.GetLengths()[3]
-            << " -W " << xDesc.GetLengths()[4]
-            << " -k " << wDesc.GetLengths()[0]
-            << " --fil_d " << wDesc.GetLengths()[2]
-            << " -y " << wDesc.GetLengths()[3]
-            << " -x " << wDesc.GetLengths()[4]
-            << " --pad_d " << convDesc.GetConvPads()[0]
-            << " -p " << convDesc.GetConvPads()[1]
-            << " -q " << convDesc.GetConvPads()[2]
-            << " --conv_stride_d " << convDesc.GetConvStrides()[0]
-            << " -u " << convDesc.GetConvStrides()[1]
-            << " -v " << convDesc.GetConvStrides()[2]
-            << " --dilation_d " << convDesc.GetConvDilations()[0]
-            << " -l " << convDesc.GetConvDilations()[1]
-            << " -j " << convDesc.GetConvDilations()[2]
-            << " --spatial_dim 3"; // clang-format on
-        std::string x_layout = xDesc.GetLayout("NCDHW");
-        std::string w_layout = wDesc.GetLayout("NCDHW");
-        std::string y_layout = yDesc.GetLayout("NCDHW");
-        if(x_layout != "NCDHW")
-        {
-            ss << " --in_layout " << x_layout;
-        }
-        if(w_layout != "NCDHW")
-        {
-            ss << " --fil_layout " << w_layout;
-        }
-        if(y_layout != "NCDHW")
-        {
-            ss << " --out_layout " << y_layout;
-        }
-    }
-    ss << " -m " << (convDesc.mode == 1 ? "trans" : "conv") // clang-format off
-        << " -g " << convDesc.group_count
-        << " -F " << std::to_string(static_cast<int>(conv_dir))
-        << " -t 1"; // clang-format on
-    if(xDesc.GetType() == miopenInt8x4)
-        ss << " -Z 1";
-    if(immediate_mode_solver_id.has_value())
-    {
-        ss << " -S " << *immediate_mode_solver_id;
-    }
-
-    return ss.str();
 }
 
 namespace miopen {
 namespace debug {
 
+MIOPEN_EXPORT
 void LogCmdConvolution(const miopen::TensorDescriptor& x,
                        const miopen::TensorDescriptor& w,
                        const miopen::ConvolutionDescriptor& conv,
@@ -538,6 +395,7 @@ void LogCmdConvolution(const miopen::TensorDescriptor& x,
     }
 }
 
+MIOPEN_EXPORT
 void LogCmdFindConvolution(const miopen::TensorDescriptor& x,
                            const miopen::TensorDescriptor& w,
                            const miopen::ConvolutionDescriptor& conv,
@@ -552,17 +410,7 @@ void LogCmdFindConvolution(const miopen::TensorDescriptor& x,
     }
 }
 
-static miopenProblemDirection_t CmdArgToDirection(ConvDirection direction)
-{
-    switch(direction)
-    {
-    case ConvDirection::Fwd: return miopenProblemDirectionForward;
-    case ConvDirection::Bwd: return miopenProblemDirectionBackward;
-    case ConvDirection::WrW: return miopenProblemDirectionBackwardWeights;
-    };
-    MIOPEN_THROW(miopenStatusInternalError);
-}
-
+MIOPEN_EXPORT
 void LogCmdConvolution(const miopenTensorDescriptor_t& xDesc,
                        const miopenTensorDescriptor_t& wDesc,
                        const miopenConvolutionDescriptor_t& convDesc,
@@ -579,6 +427,7 @@ void LogCmdConvolution(const miopenTensorDescriptor_t& xDesc,
     }
 }
 
+MIOPEN_EXPORT
 void LogCmdFindConvolution(const miopenTensorDescriptor_t& xDesc,
                            const miopenTensorDescriptor_t& wDesc,
                            const miopenConvolutionDescriptor_t& convDesc,
@@ -630,9 +479,11 @@ miopenFindConvolutionForwardAlgorithm(miopenHandle_t handle,
                         workSpaceSize,
                         exhaustiveSearch);
 
-    miopen::debug::LogCmdFindConvolution(xDesc, wDesc, convDesc, yDesc, ConvDirection::Fwd, false);
-    /// workaround for previous trans conv logic
+    miopen::debug::LogCmdFindConvolution(
+        xDesc, wDesc, convDesc, yDesc, miopen::debug::ConvDirection::Fwd, false);
+
     if(miopen::deref(convDesc).mode == miopenTranspose)
+    {
         return miopen::try_([&] {
             miopen::deref(convDesc).FindConvBwdDataAlgorithm(miopen::deref(handle),
                                                              miopen::deref(xDesc),
@@ -655,6 +506,7 @@ miopenFindConvolutionForwardAlgorithm(miopenHandle_t handle,
                     static_cast<miopenConvFwdAlgorithm_t>(perfResults[i].bwd_data_algo);
             }
         });
+    }
 
     return miopen::try_([&] {
         miopen::deref(convDesc).FindConvFwdAlgorithm(miopen::deref(handle),
@@ -701,10 +553,11 @@ extern "C" miopenStatus_t miopenConvolutionForward(miopenHandle_t handle,
                         y,
                         workSpace,
                         workSpaceSize);
-    miopen::debug::LogCmdConvolution(xDesc, wDesc, convDesc, yDesc, ConvDirection::Fwd, false);
+    miopen::debug::LogCmdConvolution(
+        xDesc, wDesc, convDesc, yDesc, miopen::debug::ConvDirection::Fwd, false);
 
-    /// workaround for previous trans conv logic
     if(miopen::deref(convDesc).mode == miopenTranspose)
+    {
         return miopen::try_([&] {
             // It is guaranteed that enum values are equal, see conv_algo_name.cpp
             const auto algo_trans = static_cast<miopenConvBwdDataAlgorithm_t>(algo);
@@ -721,6 +574,7 @@ extern "C" miopenStatus_t miopenConvolutionForward(miopenHandle_t handle,
                                                             DataCast(workSpace),
                                                             workSpaceSize);
         });
+    }
 
     return miopen::try_([&] {
         miopen::deref(convDesc).ConvolutionForward(miopen::deref(handle),
@@ -796,8 +650,10 @@ static inline void ReturnSolutions(const std::vector<miopenConvSolution_t>& solu
     if(solution_count_ret != nullptr)
         *solution_count_ret = solutions.size();
     if(solutions_ret != nullptr)
+    {
         for(auto i = 0; i < solutions.size(); ++i)
             solutions_ret[i] = solutions[i];
+    }
 }
 
 extern "C" miopenStatus_t
@@ -833,22 +689,26 @@ miopenConvolutionForwardGetSolutionWorkspaceSize(miopenHandle_t handle,
                                                  const uint64_t solution_id,
                                                  size_t* workSpaceSize)
 {
-    MIOPEN_LOG_FUNCTION(handle, wDesc, xDesc, convDesc, yDesc, solution_id, workSpaceSize);
+    MIOPEN_LOG_FUNCTION(handle, wDesc, xDesc, convDesc, yDesc, solution_id);
     return miopen::try_([&] {
         if(miopen::deref(convDesc).mode == miopenTranspose)
+        {
             *workSpaceSize =
                 miopen::deref(convDesc).GetBackwardSolutionWorkspaceSize(miopen::deref(handle),
                                                                          miopen::deref(xDesc),
                                                                          miopen::deref(wDesc),
                                                                          miopen::deref(yDesc),
                                                                          solution_id);
+        }
         else
+        {
             *workSpaceSize =
                 miopen::deref(convDesc).GetForwardSolutionWorkspaceSize(miopen::deref(handle),
                                                                         miopen::deref(wDesc),
                                                                         miopen::deref(xDesc),
                                                                         miopen::deref(yDesc),
                                                                         solution_id);
+        }
     });
 }
 
@@ -884,10 +744,12 @@ miopenConvolutionForwardImmediate(miopenHandle_t handle,
 {
     MIOPEN_LOG_FUNCTION(
         handle, wDesc, w, xDesc, x, convDesc, yDesc, y, workSpace, workSpaceSize, solution_id);
-    miopen::debug::LogCmdConvolution(xDesc, wDesc, convDesc, yDesc, ConvDirection::Fwd, true);
+    miopen::debug::LogCmdConvolution(
+        xDesc, wDesc, convDesc, yDesc, miopen::debug::ConvDirection::Fwd, true);
 
     return miopen::try_([&] {
         if(miopen::deref(convDesc).mode == miopenTranspose)
+        {
             miopen::deref(convDesc).ConvolutionBackwardImmediate(miopen::deref(handle),
                                                                  miopen::deref(xDesc),
                                                                  DataCast(x),
@@ -898,7 +760,9 @@ miopenConvolutionForwardImmediate(miopenHandle_t handle,
                                                                  DataCast(workSpace),
                                                                  workSpaceSize,
                                                                  solution_id);
+        }
         else
+        {
             miopen::deref(convDesc).ConvolutionForwardImmediate(miopen::deref(handle),
                                                                 miopen::deref(wDesc),
                                                                 DataCast(w),
@@ -909,6 +773,7 @@ miopenConvolutionForwardImmediate(miopenHandle_t handle,
                                                                 DataCast(workSpace),
                                                                 workSpaceSize,
                                                                 solution_id);
+        }
     });
 }
 
@@ -940,7 +805,7 @@ miopenConvolutionBackwardDataGetSolution(miopenHandle_t handle,
                                          size_t* solutionCount,
                                          miopenConvSolution_t* solutions)
 {
-    MIOPEN_LOG_FUNCTION(handle, dyDesc, wDesc, convDesc, dxDesc, maxSolutionCount, solutionCount);
+    MIOPEN_LOG_FUNCTION(handle, dyDesc, wDesc, convDesc, dxDesc, maxSolutionCount);
     return miopen::try_([&] {
         auto ctx               = ExecutionContext{};
         auto problem           = ProblemDescription{};
@@ -963,22 +828,26 @@ miopenConvolutionBackwardDataGetSolutionWorkspaceSize(miopenHandle_t handle,
                                                       const uint64_t solution_id,
                                                       size_t* workSpaceSize)
 {
-    MIOPEN_LOG_FUNCTION(handle, dyDesc, wDesc, convDesc, dxDesc, solution_id, workSpaceSize);
+    MIOPEN_LOG_FUNCTION(handle, dyDesc, wDesc, convDesc, dxDesc, solution_id);
     return miopen::try_([&] {
         if(miopen::deref(convDesc).mode == miopenTranspose)
+        {
             *workSpaceSize =
                 miopen::deref(convDesc).GetForwardSolutionWorkspaceSize(miopen::deref(handle),
                                                                         miopen::deref(wDesc),
                                                                         miopen::deref(dyDesc),
                                                                         miopen::deref(dxDesc),
                                                                         solution_id);
+        }
         else
+        {
             *workSpaceSize =
                 miopen::deref(convDesc).GetBackwardSolutionWorkspaceSize(miopen::deref(handle),
                                                                          miopen::deref(dyDesc),
                                                                          miopen::deref(wDesc),
                                                                          miopen::deref(dxDesc),
                                                                          solution_id);
+        }
     });
 }
 
@@ -1014,9 +883,11 @@ miopenConvolutionBackwardDataImmediate(miopenHandle_t handle,
 {
     MIOPEN_LOG_FUNCTION(
         handle, dyDesc, wDesc, convDesc, dxDesc, workSpace, workSpaceSize, solution_id);
-    miopen::debug::LogCmdConvolution(dxDesc, wDesc, convDesc, dyDesc, ConvDirection::Bwd, true);
+    miopen::debug::LogCmdConvolution(
+        dxDesc, wDesc, convDesc, dyDesc, miopen::debug::ConvDirection::Bwd, true);
     return miopen::try_([&] {
         if(miopen::deref(convDesc).mode == miopenTranspose)
+        {
             miopen::deref(convDesc).ConvolutionForwardImmediate(miopen::deref(handle),
                                                                 miopen::deref(wDesc),
                                                                 DataCast(w),
@@ -1027,7 +898,9 @@ miopenConvolutionBackwardDataImmediate(miopenHandle_t handle,
                                                                 DataCast(workSpace),
                                                                 workSpaceSize,
                                                                 solution_id);
+        }
         else
+        {
             miopen::deref(convDesc).ConvolutionBackwardImmediate(miopen::deref(handle),
                                                                  miopen::deref(dyDesc),
                                                                  DataCast(dy),
@@ -1038,6 +911,7 @@ miopenConvolutionBackwardDataImmediate(miopenHandle_t handle,
                                                                  DataCast(workSpace),
                                                                  workSpaceSize,
                                                                  solution_id);
+        }
     });
 }
 
@@ -1049,7 +923,7 @@ miopenConvolutionBackwardWeightsGetSolutionCount(miopenHandle_t handle,
                                                  const miopenTensorDescriptor_t dwDesc,
                                                  size_t* solutionCount)
 {
-    MIOPEN_LOG_FUNCTION(handle, dyDesc, xDesc, convDesc, dwDesc, solutionCount);
+    MIOPEN_LOG_FUNCTION(handle, dyDesc, xDesc, convDesc, dwDesc);
     return miopen::try_([&] {
         auto ctx               = ExecutionContext{};
         auto problem           = ProblemDescription{};
@@ -1069,7 +943,7 @@ miopenConvolutionBackwardWeightsGetSolution(miopenHandle_t handle,
                                             size_t* solutionCount,
                                             miopenConvSolution_t* solutions)
 {
-    MIOPEN_LOG_FUNCTION(handle, dyDesc, xDesc, convDesc, dwDesc, maxSolutionCount, solutionCount);
+    MIOPEN_LOG_FUNCTION(handle, dyDesc, xDesc, convDesc, dwDesc, maxSolutionCount);
     return miopen::try_([&] {
         auto ctx               = ExecutionContext{};
         auto problem           = ProblemDescription{};
@@ -1092,22 +966,26 @@ extern "C" miopenStatus_t miopenConvolutionBackwardWeightsGetSolutionWorkspaceSi
     const uint64_t solution_id,
     size_t* workSpaceSize)
 {
-    MIOPEN_LOG_FUNCTION(handle, dyDesc, xDesc, convDesc, dwDesc, solution_id, workSpaceSize);
+    MIOPEN_LOG_FUNCTION(handle, dyDesc, xDesc, convDesc, dwDesc, solution_id);
     return miopen::try_([&] {
         if(miopen::deref(convDesc).mode == miopenTranspose)
+        {
             *workSpaceSize =
                 miopen::deref(convDesc).GetWrwSolutionWorkspaceSize(miopen::deref(handle),
                                                                     miopen::deref(xDesc),
                                                                     miopen::deref(dyDesc),
                                                                     miopen::deref(dwDesc),
                                                                     solution_id);
+        }
         else
+        {
             *workSpaceSize =
                 miopen::deref(convDesc).GetWrwSolutionWorkspaceSize(miopen::deref(handle),
                                                                     miopen::deref(dyDesc),
                                                                     miopen::deref(xDesc),
                                                                     miopen::deref(dwDesc),
                                                                     solution_id);
+        }
     });
 }
 
@@ -1143,9 +1021,11 @@ miopenConvolutionBackwardWeightsImmediate(miopenHandle_t handle,
 {
     MIOPEN_LOG_FUNCTION(
         handle, dyDesc, dy, xDesc, x, convDesc, dwDesc, dw, workSpace, workSpaceSize, solution_id);
-    miopen::debug::LogCmdConvolution(xDesc, dwDesc, convDesc, dyDesc, ConvDirection::WrW, true);
+    miopen::debug::LogCmdConvolution(
+        xDesc, dwDesc, convDesc, dyDesc, miopen::debug::ConvDirection::WrW, true);
     return miopen::try_([&] {
         if(miopen::deref(convDesc).mode == miopenTranspose)
+        {
             miopen::deref(convDesc).ConvolutionWrwImmediate(miopen::deref(handle),
                                                             miopen::deref(xDesc),
                                                             DataCast(x),
@@ -1156,7 +1036,9 @@ miopenConvolutionBackwardWeightsImmediate(miopenHandle_t handle,
                                                             DataCast(workSpace),
                                                             workSpaceSize,
                                                             solution_id);
+        }
         else
+        {
             miopen::deref(convDesc).ConvolutionWrwImmediate(miopen::deref(handle),
                                                             miopen::deref(dyDesc),
                                                             DataCast(dy),
@@ -1167,6 +1049,7 @@ miopenConvolutionBackwardWeightsImmediate(miopenHandle_t handle,
                                                             DataCast(workSpace),
                                                             workSpaceSize,
                                                             solution_id);
+        }
     });
 }
 
@@ -1203,9 +1086,10 @@ miopenFindConvolutionBackwardDataAlgorithm(miopenHandle_t handle,
                         exhaustiveSearch);
 
     miopen::debug::LogCmdFindConvolution(
-        dxDesc, wDesc, convDesc, dyDesc, ConvDirection::Bwd, false);
-    /// workaround for previous trans conv logic
+        dxDesc, wDesc, convDesc, dyDesc, miopen::debug::ConvDirection::Bwd, false);
+
     if(miopen::deref(convDesc).mode == miopenTranspose)
+    {
         return miopen::try_([&] {
             miopen::deref(convDesc).FindConvFwdAlgorithm(miopen::deref(handle),
                                                          miopen::deref(dyDesc),
@@ -1228,6 +1112,7 @@ miopenFindConvolutionBackwardDataAlgorithm(miopenHandle_t handle,
                     static_cast<miopenConvBwdDataAlgorithm_t>(perfResults[i].fwd_algo);
             }
         });
+    }
 
     return miopen::try_([&] {
         miopen::deref(convDesc).FindConvBwdDataAlgorithm(miopen::deref(handle),
@@ -1275,10 +1160,11 @@ miopenConvolutionBackwardData(miopenHandle_t handle,
                         dx,
                         workSpace,
                         workSpaceSize);
-    miopen::debug::LogCmdConvolution(dxDesc, wDesc, convDesc, dyDesc, ConvDirection::Bwd, false);
+    miopen::debug::LogCmdConvolution(
+        dxDesc, wDesc, convDesc, dyDesc, miopen::debug::ConvDirection::Bwd, false);
 
-    /// workaround for previous trans conv logic
     if(miopen::deref(convDesc).mode == miopenTranspose)
+    {
         return miopen::try_([&] {
             // It is guaranteed that enum values are equal, see conv_algo_name.cpp
             const auto algo_trans = static_cast<miopenConvFwdAlgorithm_t>(algo);
@@ -1295,6 +1181,7 @@ miopenConvolutionBackwardData(miopenHandle_t handle,
                                                        DataCast(workSpace),
                                                        workSpaceSize);
         });
+    }
 
     return miopen::try_([&] {
         miopen::deref(convDesc).ConvolutionBackwardData(miopen::deref(handle),
@@ -1320,8 +1207,7 @@ miopenConvolutionBackwardDataGetWorkSpaceSize(miopenHandle_t handle,
                                               const miopenTensorDescriptor_t dxDesc,
                                               size_t* workSpaceSize)
 {
-
-    MIOPEN_LOG_FUNCTION(handle, dyDesc, wDesc, convDesc, dxDesc, workSpaceSize);
+    MIOPEN_LOG_FUNCTION(handle, dyDesc, wDesc, convDesc, dxDesc);
     return miopen::try_([&] {
         auto ctx               = ExecutionContext{};
         auto problem           = ProblemDescription{};
@@ -1338,8 +1224,7 @@ miopenConvolutionBackwardWeightsGetWorkSpaceSize(miopenHandle_t handle,
                                                  const miopenTensorDescriptor_t dwDesc,
                                                  size_t* workSpaceSize)
 {
-
-    MIOPEN_LOG_FUNCTION(handle, dyDesc, xDesc, convDesc, dwDesc, workSpaceSize);
+    MIOPEN_LOG_FUNCTION(handle, dyDesc, xDesc, convDesc, dwDesc);
     return miopen::try_([&] {
         auto ctx               = ExecutionContext{};
         auto problem           = ProblemDescription{};
@@ -1380,18 +1265,16 @@ miopenFindConvolutionBackwardWeightsAlgorithm(miopenHandle_t handle,
                         workSpaceSize,
                         exhaustiveSearch);
     miopen::debug::LogCmdFindConvolution(
-        xDesc, dwDesc, convDesc, dyDesc, ConvDirection::WrW, false);
+        xDesc, dwDesc, convDesc, dyDesc, miopen::debug::ConvDirection::WrW, false);
 
     return miopen::try_([&] {
+        const auto trans = (miopen::deref(convDesc).mode == miopenTranspose);
         miopen::deref(convDesc).FindConvBwdWeightsAlgorithm(
             miopen::deref(handle),
-            /// workaround for previous trans conv logic
-            miopen::deref(convDesc).mode == miopenTranspose ? miopen::deref(xDesc)
-                                                            : miopen::deref(dyDesc),
-            miopen::deref(convDesc).mode == miopenTranspose ? DataCast(x) : DataCast(dy),
-            miopen::deref(convDesc).mode == miopenTranspose ? miopen::deref(dyDesc)
-                                                            : miopen::deref(xDesc),
-            miopen::deref(convDesc).mode == miopenTranspose ? DataCast(dy) : DataCast(x),
+            trans ? miopen::deref(xDesc) : miopen::deref(dyDesc),
+            trans ? DataCast(x) : DataCast(dy),
+            trans ? miopen::deref(dyDesc) : miopen::deref(xDesc),
+            trans ? DataCast(dy) : DataCast(x),
             miopen::deref(dwDesc),
             DataCast(dw),
             requestAlgoCount,
@@ -1432,19 +1315,18 @@ miopenConvolutionBackwardWeights(miopenHandle_t handle,
                         dw,
                         workSpace,
                         workSpaceSize);
-    miopen::debug::LogCmdConvolution(xDesc, dwDesc, convDesc, dyDesc, ConvDirection::WrW, false);
+    miopen::debug::LogCmdConvolution(
+        xDesc, dwDesc, convDesc, dyDesc, miopen::debug::ConvDirection::WrW, false);
 
     return miopen::try_([&] {
+        const auto trans = (miopen::deref(convDesc).mode == miopenTranspose);
         miopen::deref(convDesc).ConvolutionBackwardWeights(
             miopen::deref(handle),
             alpha,
-            /// workaround for previous trans conv logic
-            miopen::deref(convDesc).mode == miopenTranspose ? miopen::deref(xDesc)
-                                                            : miopen::deref(dyDesc),
-            miopen::deref(convDesc).mode == miopenTranspose ? DataCast(x) : DataCast(dy),
-            miopen::deref(convDesc).mode == miopenTranspose ? miopen::deref(dyDesc)
-                                                            : miopen::deref(xDesc),
-            miopen::deref(convDesc).mode == miopenTranspose ? DataCast(dy) : DataCast(x),
+            trans ? miopen::deref(xDesc) : miopen::deref(dyDesc),
+            trans ? DataCast(x) : DataCast(dy),
+            trans ? miopen::deref(dyDesc) : miopen::deref(xDesc),
+            trans ? DataCast(dy) : DataCast(x),
             algo,
             beta,
             miopen::deref(dwDesc),
@@ -1493,7 +1375,7 @@ extern "C" miopenStatus_t miopenGetConvolutionAttribute(miopenConvolutionDescrip
                                                         const miopenConvolutionAttrib_t attr,
                                                         int* const value)
 {
-    MIOPEN_LOG_FUNCTION(convDesc, attr, value);
+    MIOPEN_LOG_FUNCTION(convDesc, attr);
     return miopen::try_(
         [&] { miopen::deref(value) = miopen::deref(convDesc).attribute.Get(attr); });
 }
