@@ -52,15 +52,10 @@
 #include <chrono>
 #include <unordered_map>
 
-namespace boost {
-namespace filesystem {
-class path;
-} // namespace filesystem
-} // namespace boost
+MIOPEN_DECLARE_ENV_VAR_BOOL(MIOPEN_DEBUG_DISABLE_SQL_WAL)
+MIOPEN_DECLARE_ENV_VAR_STR(MIOPEN_DEBUG_PERFDB_OVERRIDE)
 
 namespace miopen {
-MIOPEN_DECLARE_ENV_VAR(MIOPEN_DEBUG_DISABLE_SQL_WAL)
-MIOPEN_DECLARE_ENV_VAR(MIOPEN_DEBUG_PERFDB_OVERRIDE)
 
 constexpr bool InMemDb = MIOPEN_EMBED_DB;
 #if MIOPEN_ENABLE_SQLITE_BACKOFF
@@ -80,7 +75,7 @@ struct SQLiteSerializable
                            names.push_back(name);
                        });
         Derived::Visit(static_cast<const Derived&>(*this),
-                       [&](const int value, const std::string name) {
+                       [&](const int64_t value, const std::string name) {
                            std::ignore = value;
                            names.push_back(name);
                        });
@@ -97,7 +92,7 @@ struct SQLiteSerializable
                            values.push_back(value);
                        });
         Derived::Visit(static_cast<const Derived&>(*this),
-                       [&](const int value, const std::string name) {
+                       [&](const int64_t value, const std::string name) {
                            clauses.push_back("(" + name + " = ? )");
                            values.push_back(std::to_string(value));
                        });
@@ -113,7 +108,7 @@ struct SQLiteSerializable
                            values.push_back(value);
                        });
         Derived::Visit(static_cast<const Derived&>(*this),
-                       [&](const int value, const std::string name) {
+                       [&](const int64_t value, const std::string name) {
                            int_names.push_back(name);
                            values.push_back(std::to_string(value));
                        });
@@ -144,7 +139,7 @@ struct SQLiteSerializable
                        });
         std::vector<std::string> int_fields;
         Derived::Visit(static_cast<const Derived&>(*this),
-                       [&](const int value, const std::string name) {
+                       [&](const int64_t value, const std::string name) {
                            std::ignore = value;
                            int_fields.push_back(name);
                        });
@@ -230,10 +225,9 @@ public:
             dbInvalid = true;
             return;
         }
-
-        if(!is_system && !filename.empty())
+        else if(!is_system)
         {
-            auto file            = boost::filesystem::path(filename_);
+            auto file            = fs::path(filename_);
             const auto directory = file.remove_filename();
             if(directory.string().empty())
             {
@@ -241,22 +235,24 @@ public:
                 return;
             }
 
-            if(!(boost::filesystem::exists(directory)))
+            if(!fs::exists(directory))
             {
-                if(!boost::filesystem::create_directories(directory))
+                if(!fs::create_directories(directory))
                     MIOPEN_LOG_W("Unable to create a directory: " << directory);
                 else
-                    boost::filesystem::permissions(directory, boost::filesystem::all_all);
+                    fs::permissions(directory, fs::perms::all);
             }
         }
         sql = SQLite{filename_, is_system};
         if(!sql.Valid())
         {
-            bool isKDB = boost::filesystem::path(filename).extension() == ".kdb";
+            bool isKDB = fs::path(filename).extension() == ".kdb";
             dbInvalid  = true;
             filename   = "";
             if(!is_system)
+            {
                 MIOPEN_THROW(miopenStatusInternalError, "Cannot open database file:" + filename_);
+            }
             else
             {
                 const auto log_level =
@@ -268,7 +264,7 @@ public:
                             "Missing system database file: "
                             << filename_
                             << " Performance may degrade. Please follow instructions to install: "
-                               "https://github.com/ROCmSoftwarePlatform/"
+                               "https://github.com/ROCm/"
                                "MIOpen#installing-miopen-kernels-package");
                         return true;
                     }();
@@ -285,7 +281,7 @@ public:
         else
         {
             dbInvalid = false;
-            if(!is_system && !miopen::IsEnabled(MIOPEN_DEBUG_DISABLE_SQL_WAL{}))
+            if(!is_system && !miopen::IsEnabled(ENV(MIOPEN_DEBUG_DISABLE_SQL_WAL)))
             {
                 auto res = sql.Exec("PRAGMA journal_mode=WAL;");
                 if(res.empty() || res[0]["journal_mode"] != "wal")
@@ -412,8 +408,10 @@ public:
         auto stmt              = SQLite::Statement{sql, clause, vals};
         auto rc                = stmt.Step(sql);
         if(rc != SQLITE_DONE)
+        {
             MIOPEN_THROW(miopenStatusInternalError,
                          "Failed to insert config: " + sql.ErrorMessage());
+        }
         auto cnt = sql.Changes();
         MIOPEN_LOG_I2(cnt << " rows updated");
     }
@@ -429,11 +427,17 @@ public:
         {
             auto rc = stmt.Step(sql);
             if(rc == SQLITE_ROW)
+            {
                 return stmt.ColumnText(0);
+            }
             else if(rc == SQLITE_DONE)
+            {
                 return "";
+            }
             else if(rc == SQLITE_ERROR || rc == SQLITE_MISUSE)
+            {
                 MIOPEN_THROW(miopenStatusInternalError, sql.ErrorMessage());
+            }
         }
     }
     template <typename T>
@@ -442,8 +446,8 @@ public:
         if(dbInvalid)
             return boost::none;
 
-        const auto pdb_ovr = miopen::GetStringEnv(MIOPEN_DEBUG_PERFDB_OVERRIDE{});
-        if(pdb_ovr != nullptr)
+        const auto& pdb_ovr = miopen::GetStringEnv(ENV(MIOPEN_DEBUG_PERFDB_OVERRIDE));
+        if(!pdb_ovr.empty())
         {
             MIOPEN_LOG_I2("overriding tuning params with: " << pdb_ovr);
             DbRecord ovr_rec;
@@ -483,11 +487,17 @@ public:
         {
             auto rc = stmt.Step(sql);
             if(rc == SQLITE_ROW)
+            {
                 rec.SetValues(stmt.ColumnText(0), stmt.ColumnText(1));
+            }
             else if(rc == SQLITE_DONE)
+            {
                 break;
+            }
             else if(rc == SQLITE_ERROR || rc == SQLITE_MISUSE)
+            {
                 MIOPEN_THROW(miopenStatusInternalError, sql.ErrorMessage());
+            }
         }
         if(rec.GetSize() == 0)
             return boost::none;
@@ -518,7 +528,9 @@ public:
         auto stmt = SQLite::Statement{sql, query, values};
         auto rc   = stmt.Step(sql);
         if(rc == SQLITE_DONE)
+        {
             return true;
+        }
         else
         {
             const std::string msg = "Unable to remove database entry: ";
@@ -543,8 +555,10 @@ public:
             auto stmt              = SQLite::Statement{sql, clause, vals};
             auto rc                = stmt.Step(sql);
             if(rc != SQLITE_DONE)
+            {
                 MIOPEN_THROW(miopenStatusInternalError,
                              "Failed to insert config: " + sql.ErrorMessage());
+            }
             auto cnt = sql.Changes();
             MIOPEN_LOG_I2(cnt << " rows updated");
         }
