@@ -4,7 +4,7 @@
 
 namespace miopen {
 
-std::string LoadFile(const boost::filesystem::path& p) { return LoadFile(p.string()); }
+std::string LoadFile(const fs::path& p) { return LoadFile(p.string()); }
 
 std::string LoadFile(const std::string& s)
 {
