@@ -37,12 +37,15 @@
 #include <miopen/solver.hpp>
 #include <miopen/generic_search.hpp>
 
-MIOPEN_DECLARE_ENV_VAR(MIOPEN_DEBUG_CONV_DIRECT_ASM_1X1UV2_PERF_VALS)
-MIOPEN_DECLARE_ENV_VAR(MIOPEN_DEBUG_CONV_DIRECT_ASM_1X1UV2_SEARCH_OPTIMIZED)
-MIOPEN_DECLARE_ENV_VAR(MIOPEN_DEBUG_CONV_DIRECT_ASM_1X1UV2)
+MIOPEN_DECLARE_ENV_VAR_STR(MIOPEN_DEBUG_CONV_DIRECT_ASM_1X1UV2_PERF_VALS)
+MIOPEN_DECLARE_ENV_VAR_BOOL(MIOPEN_DEBUG_CONV_DIRECT_ASM_1X1UV2_SEARCH_OPTIMIZED)
+MIOPEN_DECLARE_ENV_VAR_BOOL(MIOPEN_DEBUG_CONV_DIRECT_ASM_1X1UV2)
 
 namespace miopen {
 namespace solver {
+namespace conv {
+
+using ProblemDescription = miopen::conv::ProblemDescription;
 
 /// \todo Rework, factor out to separate header and use in other solvers.
 /// \todo Clarify functions semantics.
@@ -116,7 +119,7 @@ struct config_helper
 {
     config_helper(const ProblemDescription& problem, const PerformanceConfigConvAsm1x1UV2& config)
     {
-        if(problem.direction.IsForward())
+        if(problem.IsDirectionForward())
         {
             stride_w   = problem.GetKernelStrideW();
             stride_h   = problem.GetKernelStrideH();
@@ -199,7 +202,7 @@ bool PerformanceConfigConvAsm1x1UV2::SetNextValue(const ProblemDescription&)
     // Increment with wrap-around:
     do
     {
-        if(!miopen::IsDisabled(MIOPEN_DEBUG_CONV_DIRECT_ASM_1X1UV2_SEARCH_OPTIMIZED{}))
+        if(!miopen::IsDisabled(ENV(MIOPEN_DEBUG_CONV_DIRECT_ASM_1X1UV2_SEARCH_OPTIMIZED)))
         {
             if(!IncPack<16, 32, 64>(chunk_size))
                 break;
@@ -260,7 +263,7 @@ bool PerformanceConfigConvAsm1x1UV2::SetNextValue(const ProblemDescription&)
 PerformanceConfigConvAsm1x1UV2::PerformanceConfigConvAsm1x1UV2(bool spare)
     : PerformanceConfigConvAsm1x1UV2(1, 1, 1, 1, 1, 1, 1, 1, 1, 1, spare)
 {
-    if(!miopen::IsDisabled(MIOPEN_DEBUG_CONV_DIRECT_ASM_1X1UV2_SEARCH_OPTIMIZED{}))
+    if(!miopen::IsDisabled(ENV(MIOPEN_DEBUG_CONV_DIRECT_ASM_1X1UV2_SEARCH_OPTIMIZED)))
     {
         k_mult           = spare ? 1 : 8;
         chunk_size       = 16;
@@ -338,7 +341,7 @@ bool PerformanceConfigConvAsm1x1UV2::IsValid(const ProblemDescription& problem)
         return false;
     if(!(h_per_chunk <= chunk_size))
         return false;
-    if(!(k_mult * waves_k_in_group <= problem.GetOutChannels()))
+    if(!(static_cast<std::size_t>(k_mult) * waves_k_in_group <= problem.GetOutChannels()))
         return false;
 
     // cppcheck-suppress unreadVariable
@@ -386,7 +389,7 @@ bool PerformanceConfigConvAsm1x1UV2::IsValid(const ProblemDescription& problem)
     const auto c_per_wave = (problem.GetInChannels() + waves_c_in_group - 1) / waves_c_in_group;
     const auto c_per_last_wave = problem.GetInChannels() - (c_per_wave * (waves_c_in_group - 1));
 
-    if(problem.direction.IsBackwardData() && !(problem.GetOutChannels() % k_mult == 0))
+    if(problem.IsDirectionBackwardData() && !(problem.GetOutChannels() % k_mult == 0))
         return false;
 
     {
@@ -407,9 +410,9 @@ bool PerformanceConfigConvAsm1x1UV2::IsValid(const ProblemDescription& problem)
                        1,
                        GetTypeSize(problem.GetOutDataType()));
         int n_miss = n_mult * GetNPerGpr() - 1;
-        if((static_cast<long>(problem.GetInChannels()) + n_miss) * ibuf.byte_stride.nk >=
+        if((static_cast<int64_t>(problem.GetInChannels()) + n_miss) * ibuf.byte_stride.nk >=
                (1LL << 31) ||
-           (static_cast<long>(problem.GetOutChannels()) + n_miss) * obuf.byte_stride.nk >=
+           (static_cast<int64_t>(problem.GetOutChannels()) + n_miss) * obuf.byte_stride.nk >=
                (1LL << 31))
             return false;
     }
@@ -418,8 +421,8 @@ bool PerformanceConfigConvAsm1x1UV2::IsValid(const ProblemDescription& problem)
 
 void PerformanceConfigConvAsm1x1UV2::HeuristicInit(const ProblemDescription& problem)
 {
-    int c_check   = problem.direction.IsForward() ? problem.GetInChannels() : 0;
-    int k_check   = problem.direction.IsForward() ? 0 : problem.GetInChannels();
+    int c_check   = problem.IsDirectionForward() ? problem.GetInChannels() : 0;
+    int k_check   = problem.IsDirectionForward() ? 0 : problem.GetInChannels();
     chunk_size    = 16;
     dwords_per_ld = 1;
     c_mult        = (c_check % 2 == 0) ? 2 : ((c_check % 3 == 0) ? 3 : 1);
@@ -460,7 +463,7 @@ void PerformanceConfigConvAsm1x1UV2::HeuristicInit(const ProblemDescription& pro
 }
 
 PerformanceConfigConvAsm1x1UV2
-ConvAsm1x1UV2::GetDefaultPerformanceConfig(const ConvolutionContext&,
+ConvAsm1x1UV2::GetDefaultPerformanceConfig(const ExecutionContext&,
                                            const ProblemDescription& problem) const
 {
     PerformanceConfigConvAsm1x1UV2 pp;
@@ -469,17 +472,17 @@ ConvAsm1x1UV2::GetDefaultPerformanceConfig(const ConvolutionContext&,
     return pp;
 }
 
-bool ConvAsm1x1UV2::IsValidPerformanceConfig(const ConvolutionContext&,
+bool ConvAsm1x1UV2::IsValidPerformanceConfig(const ExecutionContext&,
                                              const ProblemDescription& problem,
                                              const PerformanceConfigConvAsm1x1UV2& config) const
 {
     return config.IsValidValue() && config.IsValid(problem);
 }
 
-bool ConvAsm1x1UV2::IsApplicable(const ConvolutionContext& ctx,
+bool ConvAsm1x1UV2::IsApplicable(const ExecutionContext& ctx,
                                  const ProblemDescription& problem) const
 {
-    if(miopen::IsDisabled(MIOPEN_DEBUG_CONV_DIRECT_ASM_1X1UV2{}))
+    if(miopen::IsDisabled(ENV(MIOPEN_DEBUG_CONV_DIRECT_ASM_1X1UV2)))
         return false;
     if(ThisSolverIsDeprecatedStatic::IsDisabled(ctx))
         return false;
@@ -487,7 +490,11 @@ bool ConvAsm1x1UV2::IsApplicable(const ConvolutionContext& ctx,
         return false;
     if(!problem.Is2d())
         return false;
-    if(!(problem.direction.IsForward() || problem.direction.IsBackwardData()))
+    if(!(problem.IsDirectionForward() || problem.IsDirectionBackwardData()))
+        return false;
+    if(problem.HasNonPackedTensors())
+        return false;
+    if(problem.HasAtLeastOne64BitTensor())
         return false;
     if(problem.IsAsymmetricPadH() || problem.IsAsymmetricPadW())
         return false;
@@ -496,19 +503,21 @@ bool ConvAsm1x1UV2::IsApplicable(const ConvolutionContext& ctx,
     if(!problem.IsFp32())
         return false;
 
+    if(problem.IsTensorsCasted())
+        return false;
+
     const auto target = ctx.GetStream().GetTargetProperties();
     if(target.Xnack() && *target.Xnack())
         return false;
 
     const std::string name = ctx.GetStream().GetDeviceName();
     if(name.find("gfx8") == std::string::npos && name.find("gfx9") == std::string::npos)
-    {
         return false;
-    }
     if(!problem.IsLayoutDefault())
-    {
         return false;
-    }
+
+    if(problem.IsTensorsCasted() || problem.IsFp8() || problem.IsBfp8())
+        return false;
 
     const auto elements_in_dword = 4 / GetTypeSize(problem.GetInDataType());
     // clang-format off
@@ -537,13 +546,13 @@ bool ConvAsm1x1UV2::IsApplicable(const ConvolutionContext& ctx,
     }
 
     // Check limits:
-    auto h_w = static_cast<long>(problem.GetInHeight()) * problem.GetInWidth();
-    const auto r_s     = static_cast<long>(problem.GetWeightsHeight()) * problem.GetWeightsWidth();
-    const auto c_h_w   = static_cast<long>(problem.GetInChannels()) * h_w;  // C*H*W
-    const auto k_h_w   = static_cast<long>(problem.GetOutChannels()) * h_w; // K*H*W
-    const auto n_c_h_w = static_cast<long>(problem.GetBatchSize()) * c_h_w; // N*C*H*W
-    const auto n_k_h_w = static_cast<long>(problem.GetBatchSize()) * k_h_w; // N*K*H*W
-    const auto c_k_r_s = static_cast<long>(problem.GetInChannels()) * problem.GetOutChannels() * r_s; // C*K*R*S
+    const auto h_w     = problem.GetInHeight() * problem.GetInWidth();
+    const auto r_s     = problem.GetWeightsHeight() * problem.GetWeightsWidth();
+    const auto c_h_w   = problem.GetInChannels() * h_w;  // C*H*W
+    const auto k_h_w   = problem.GetOutChannels() * h_w; // K*H*W
+    const auto n_c_h_w = problem.GetBatchSize() * c_h_w; // N*C*H*W
+    const auto n_k_h_w = problem.GetBatchSize() * k_h_w; // N*K*H*W
+    const auto c_k_r_s = problem.GetInChannels() * problem.GetOutChannels() * r_s; // C*K*R*S
     ok = problem.GetBatchSize() < std::pow(2, 16)       // -n   N batch_size
          && problem.GetInChannels() < std::pow(2, 16)   // -c   C input_channels
          && problem.GetOutChannels() < std::pow(2, 16)  // -k   K output_channels
@@ -579,16 +588,16 @@ bool ConvAsm1x1UV2::IsApplicable(const ConvolutionContext& ctx,
         const int eurictic_init_max_chunk_size = 16;
         const int n_miss = eurictic_init_min_n_mult * (64 / eurictic_init_max_chunk_size) - 1;
 
-        if((static_cast<long>(config.GetInChannels()) + n_miss) * ibuf.byte_stride.nk >=
+        if((static_cast<int64_t>(config.GetInChannels()) + n_miss) * ibuf.byte_stride.nk >=
                (1LL << 31) ||
-           (static_cast<long>(config.GetOutChannels()) + n_miss) * obuf.byte_stride.nk >=
+           (static_cast<int64_t>(config.GetOutChannels()) + n_miss) * obuf.byte_stride.nk >=
                (1LL << 31))
             ok = false;
     }
     return ok;
 }
 
-ConvSolution ConvAsm1x1UV2::GetSolution(const ConvolutionContext& ctx,
+ConvSolution ConvAsm1x1UV2::GetSolution(const ExecutionContext& ctx,
                                         const ProblemDescription& problem,
                                         const PerformanceConfigConvAsm1x1UV2& config) const
 {
@@ -602,24 +611,19 @@ ConvSolution ConvAsm1x1UV2::GetSolution(const ConvolutionContext& ctx,
 
     PerformanceConfigConvAsm1x1UV2 fromEnv;
     {
-        std::string s;
-        const auto p_asciz = miopen::GetStringEnv(MIOPEN_DEBUG_CONV_DIRECT_ASM_1X1UV2_PERF_VALS{});
-        if(p_asciz != nullptr)
+        const auto& s = miopen::GetStringEnv(ENV(MIOPEN_DEBUG_CONV_DIRECT_ASM_1X1UV2_PERF_VALS));
+        if(!s.empty()) // else nothing to parse.
         {
-            s = std::string(p_asciz);
-            if(!s.empty()) // else nothing to parse.
+            if(!fromEnv.Deserialize(s) || !fromEnv.IsValidValue())
             {
-                if(!fromEnv.Deserialize(s) || !fromEnv.IsValidValue())
-                {
-                    MIOPEN_LOG_E("MIOPEN_DEBUG_CONV_DIRECT_ASM_1X1UV2_PERF_VALS: "
-                                 "Bad format or invalid for the problem config: "
-                                 << s);
-                }
-                else
-                {
-                    MIOPEN_LOG_I("Overridden from env: " << fromEnv.ToString());
-                    pcfg = &fromEnv;
-                }
+                MIOPEN_LOG_E("MIOPEN_DEBUG_CONV_DIRECT_ASM_1X1UV2_PERF_VALS: "
+                             "Bad format or invalid for the problem config: "
+                             << s);
+            }
+            else
+            {
+                MIOPEN_LOG_I("Overridden from env: " << fromEnv.ToString());
+                pcfg = &fromEnv;
             }
         }
     }
@@ -647,7 +651,7 @@ ConvSolution ConvAsm1x1UV2::GetSolution(const ConvolutionContext& ctx,
     GenerateClangDefsym(options, "wei_w", problem.GetWeightsWidth());          // S
     GenerateClangDefsym(options, "pad_h", problem.GetPadH());
     GenerateClangDefsym(options, "pad_w", problem.GetPadW());
-    GenerateClangDefsym(options, "weights_layout", problem.direction.IsForward() ? 0 : 1);
+    GenerateClangDefsym(options, "weights_layout", problem.IsDirectionForward() ? 0 : 1);
 
     GenerateClangDefsym(options, "vec_c_in", 1);
     GenerateClangDefsym(options, "vec_k_out", 1);
@@ -673,7 +677,7 @@ ConvSolution ConvAsm1x1UV2::GetSolution(const ConvolutionContext& ctx,
                    1,
                    data_len);
     // cppcheck-suppress unreadVariable
-    buff_info fbuf(problem.direction.IsForward() ? MemLayout::NCHW : MemLayout::CNHW,
+    buff_info fbuf(problem.IsDirectionForward() ? MemLayout::NCHW : MemLayout::CNHW,
                    problem.GetOutChannels(),
                    problem.GetInChannels(),
                    1,
@@ -742,18 +746,20 @@ ConvSolution ConvAsm1x1UV2::GetSolution(const ConvolutionContext& ctx,
     {
         int N, C, H, W, K, n_groups;
         GetCompiledInParameters(ctx, problem, &N, &C, &H, &W, &K, &n_groups);
-        result.invoker_factory = conv::MakeGcnAsm1x1UInvokerFactory(N, C, H, W, K, n_groups);
+        result.invoker_factory =
+            miopen::conv::MakeGcnAsm1x1UInvokerFactory(N, C, H, W, K, n_groups);
     }
 
     return result;
 }
 
-PerformanceConfigConvAsm1x1UV2 ConvAsm1x1UV2::Search(const ConvolutionContext& ctx,
+PerformanceConfigConvAsm1x1UV2 ConvAsm1x1UV2::Search(const ExecutionContext& ctx,
                                                      const ProblemDescription& problem,
                                                      const AnyInvokeParams& invoke_ctx) const
 {
     return GenericSearch(*this, ctx, problem, invoke_ctx);
 }
 
+} // namespace conv
 } // namespace solver
 } // namespace miopen
