@@ -33,7 +33,11 @@
 #include <miopen/legacy_exhaustive_search.hpp>
 #include <miopen/bfloat16.hpp>
 #include <miopen/fusion/fusion_invoke_params.hpp>
+#if !defined(_WIN32)
+#include <half/half.hpp>
+#else
 #include <half.hpp>
+#endif
 
 #ifdef max
 #undef max
@@ -45,12 +49,15 @@
 
 namespace miopen {
 namespace solver {
+namespace conv {
+
+using ProblemDescription = miopen::conv::ProblemDescription;
 
 /*
  * select default configuration if a known configuration has not been found.
  */
 LegacyPerformanceConfig ConvOclDirectFwdLegacyExhaustiveSearch::GetDefaultPerformanceConfig(
-    const ConvolutionContext& ctx, const ProblemDescription& problem) const
+    const ExecutionContext& ctx, const ProblemDescription& problem) const
 {
     //
     LegacyPerformanceConfig result{};
@@ -83,7 +90,7 @@ LegacyPerformanceConfig ConvOclDirectFwdLegacyExhaustiveSearch::GetDefaultPerfor
     {
 
         // version
-        if(problem.GetInDataType() == miopenFloat && problem.direction.IsForward() &&
+        if(problem.GetInDataType() == miopenFloat && problem.IsDirectionForward() &&
            problem.GetInChannels() % 16 == 0 && problem.GetOutChannels() % 16 == 0)
         {
             result.n_in_data_tiles = 128;
@@ -103,7 +110,7 @@ LegacyPerformanceConfig ConvOclDirectFwdLegacyExhaustiveSearch::GetDefaultPerfor
 
             if(problem.GetPadW() > 0 || problem.GetKernelStrideW() > 1)
             {
-                if(problem.direction.IsForward())
+                if(problem.IsDirectionForward())
                 {
                     result.out_pix_tile0 = (problem.GetOutWidth() & 1) != 0 ? 1 : 2;
                 }
@@ -138,7 +145,7 @@ static int MeasurePerfConfig(const Handle& handle,
                              ConstData_t wei_ocl_buf,
                              ConstData_t bias_ocl_buf,
                              double& processing_time,
-                             const ConvolutionContext& ctx,
+                             const ExecutionContext& ctx,
                              const ProblemDescription& problem,
                              const LegacyPerformanceConfig& config)
 {
@@ -163,13 +170,11 @@ static int MeasurePerfConfig(const Handle& handle,
     {
         return 1;
     }
-#if !MIOPEN_ALLOC_BUFFERS
     if(problem.GetBias() && bias_ocl_buf == nullptr)
     {
         MIOPEN_LOG_WE("Legacy search: Bias buffer required");
         return 2;
     }
-#endif
 
     MIOPEN_LOG_I2("Trying " << config);
     const auto kernel_params     = kernel_search_result.construction_params[0];
@@ -209,16 +214,22 @@ static int MeasurePerfConfig(const Handle& handle,
 }
 
 LegacyPerformanceConfig
-ConvOclDirectFwdLegacyExhaustiveSearch::Search(const ConvolutionContext& ctx,
+ConvOclDirectFwdLegacyExhaustiveSearch::Search(const ExecutionContext& ctx,
                                                const ProblemDescription& problem,
                                                const AnyInvokeParams& invoke_ctx) const
 {
     if(problem.IsFp16())
+    {
         return SearchImpl<half_float::half>(ctx, problem, invoke_ctx);
+    }
     else if(problem.IsFp32())
+    {
         return SearchImpl<float>(ctx, problem, invoke_ctx);
+    }
     else if(problem.IsBfp16())
+    {
         return SearchImpl<bfloat16>(ctx, problem, invoke_ctx);
+    }
     else
     {
         MIOPEN_THROW("Unsupported float_size");
@@ -227,7 +238,7 @@ ConvOclDirectFwdLegacyExhaustiveSearch::Search(const ConvolutionContext& ctx,
 
 template <typename Tgpu>
 LegacyPerformanceConfig
-ConvOclDirectFwdLegacyExhaustiveSearch::SearchImpl(const ConvolutionContext& ctx,
+ConvOclDirectFwdLegacyExhaustiveSearch::SearchImpl(const ExecutionContext& ctx,
                                                    const ProblemDescription& problem,
                                                    const AnyInvokeParams& invoke_ctx) const
 {
@@ -247,60 +258,14 @@ ConvOclDirectFwdLegacyExhaustiveSearch::SearchImpl(const ConvolutionContext& ctx
     candidate.n_in_data_tiles = 3;
     candidate.n_stacks        = 1;
 
-#if MIOPEN_ALLOC_BUFFERS
-    miopen::Handle profile_h;
-
-    // allocate input/output buffers
-    size_t bot_sz = problem.GetInSize() / sizeof(Tgpu);
-    std::vector<Tgpu> bot_sys_buf(bot_sz);
-    for(size_t i = 0; i < bot_sz; i++)
-    {
-        bot_sys_buf[i] =
-            static_cast<Tgpu>(rand() * (1.0 / RAND_MAX)); // NOLINT (concurrency-mt-unsafe)
-    }
-    auto bot_ocl_buf = profile_h.Write(bot_sys_buf);
-    auto bot_ocl_ptr = bot_ocl_buf.get();
-
-    size_t top_sz = problem.GetOutSize() / sizeof(Tgpu);
-    std::vector<Tgpu> top_sys_buf(top_sz);
-    auto top_ocl_buf = profile_h.Write(top_sys_buf);
-    auto top_ocl_ptr = top_ocl_buf.get();
-
-    size_t weights_sz = problem.GetWeightsSize() / sizeof(Tgpu);
-    std::vector<Tgpu> wei_sys_buf(weights_sz);
-    for(size_t i = 0; i < weights_sz; i++)
-    {
-        wei_sys_buf[i] = static_cast<Tgpu>((rand() * (1.0 / RAND_MAX) - 0.5) *
-                                           0.001); // NOLINT (concurrency-mt-unsafe)
-    }
-    auto wei_ocl_buf = profile_h.Write(wei_sys_buf);
-    auto wei_ocl_ptr = wei_ocl_buf.get();
-
-    std::vector<Tgpu> bias_sys_buf;
-    miopen::Allocator::ManageDataPtr bias_ocl_buf = nullptr;
-    if(problem.GetBias() != 0)
-    {
-        size_t bias_sz = problem.GetBiasSize() / sizeof(Tgpu);
-        bias_sys_buf   = std::vector<Tgpu>(bias_sz);
-        for(size_t i = 0; i < bias_sz; i++)
-        {
-            bias_sys_buf[i] =
-                static_cast<Tgpu>(rand() * (1.0 / RAND_MAX)); // NOLINT (concurrency-mt-unsafe)
-        }
-
-        bias_ocl_buf = profile_h.Write(bias_sys_buf);
-    }
-    auto bias_ocl_ptr = bias_ocl_buf.get();
-#else
     auto& profile_h           = ctx.GetStream();
-    const auto& invoke_params = invoke_ctx.CastTo<conv::DataInvokeParams>();
+    const auto& invoke_params = invoke_ctx.CastTo<miopen::conv::DataInvokeParams>();
     const auto bot_ocl_ptr    = invoke_params.tensors.in;
     const auto top_ocl_ptr    = invoke_params.tensors.out;
     const auto wei_ocl_ptr    = invoke_params.tensors.w;
     // There was no place in the source, where it has been actually set to something other than
     // nullptr.
     const auto bias_ocl_ptr = static_cast<Data_t>(nullptr);
-#endif
     AutoEnableProfiling enableProfiling{profile_h};
 
     // search loop here
@@ -320,7 +285,7 @@ ConvOclDirectFwdLegacyExhaustiveSearch::SearchImpl(const ConvolutionContext& ctx
     int out_pix_tl_cnt = 3; // out_pix_tile_sz[1];
     int n_out_tls      = 4;
     int n_in_tls       = 3;
-    int stack_cnt      = std::min(problem.GetBatchSize(), 2);
+    int stack_cnt      = std::min(problem.GetBatchSize(), static_cast<std::size_t>(2));
     int n_tile0_sz     = 4;
     int n_tile1_sz     = 4;
 
@@ -356,22 +321,21 @@ ConvOclDirectFwdLegacyExhaustiveSearch::SearchImpl(const ConvolutionContext& ctx
         report_inteval   = 5;
 
         // Add 1x1_stride : no padding support yet
-        if(problem.GetInDataType() == miopenFloat && problem.direction.IsForward() &&
+        if(problem.GetInDataType() == miopenFloat && problem.IsDirectionForward() &&
            problem.GetInChannels() % 16 == 0 && problem.GetOutChannels() % 16 == 0)
         {
 
-            // uint N_LCL_IN_MAPS = result.n_in_data_tiles;
+            // unsigned N_LCL_IN_MAPS = result.n_in_data_tiles;
             n_in_tiles_rg[0] = 0;
             n_in_tiles_rg[1] = 3;
             n_in_tls         = 4;
 
-            //					int N_LCL_OUT_MAPS = result.n_out_pix_tiles;
+            // int N_LCL_OUT_MAPS = result.n_out_pix_tiles;
             n_out_tiles_rg[0] = 4;
             n_out_tiles_rg[1] = 6;
             // 0 or 1
             out_pix_tl_cnt = 3;
-            //					uint CHEAT_SHADER_COMPILER =
-            // result.out_pix_tile0;
+            // unsigned CHEAT_SHADER_COMPILER = result.out_pix_tile0;
             out_pix_tile_sz[0] = 0;
             out_pix_tile_sz[1] = 1;
             n_out_tls          = (n_out_tiles_rg[1] - n_out_tiles_rg[0] + 1);
@@ -393,7 +357,7 @@ ConvOclDirectFwdLegacyExhaustiveSearch::SearchImpl(const ConvolutionContext& ctx
             }
             else
             {
-                if(problem.direction.IsForward())
+                if(problem.IsDirectionForward())
                 {
                     out_pix_tl_cnt = (problem.GetOutWidth() & 1) != 0 ? 1 : 2;
                 }
@@ -709,5 +673,6 @@ ConvOclDirectFwdLegacyExhaustiveSearch::SearchImpl(const ConvolutionContext& ctx
     return result;
 }
 
+} // namespace conv
 } // namespace solver
 } // namespace miopen
